@@ -51,6 +51,7 @@ std::string ExistingFile::parse(const std::string& val)
 		throw exception::BadOption("file " + val + " must exist");
 	return val;
 }
+std::string ExistingFile::toString(const std::string& val) { return val; }
 
 static string fmtshort(char c, const std::string& usage)
 {
@@ -84,6 +85,8 @@ static string manfmtlong(const std::string& c, const std::string& usage)
 		return string("\\-\\-") + c + "=\\fI" + usage + "\\fP";
 }
 
+Option::Option() : hidden(false) {}
+
 const std::string& Option::fullUsage() const
 {
 	if (m_fullUsage.empty())
