@@ -13,14 +13,27 @@ namespace fs {
 std::auto_ptr<struct stat> stat(const std::string& pathname)
 {
 	std::auto_ptr<struct stat> res(new struct stat);
-	if (::stat(pathname.c_str(), res.get()) == -1)
+	if (::stat(pathname.c_str(), res.get()) == -1) {
 		if (errno == ENOENT)
 			return std::auto_ptr<struct stat>();
 		else
 			throw wibble::exception::System("getting file information for " + pathname);
+        }
 	return res;
 }
 
+bool isDirectory(const std::string& pathname)
+{
+	struct stat st;
+	if (::stat(pathname.c_str(), &st) == -1) {
+		if (errno == ENOENT)
+			return false;
+		else
+			throw wibble::exception::System("getting file information for " + pathname);
+        }
+	return S_ISDIR(st.st_mode);
+}
+
 bool access(const std::string &s, int m)
 {
 	return ::access(s.c_str(), m) == 0;
@@ -89,6 +102,17 @@ void writeFile( const std::string &file, const std::string &data )
     out << data;
 }
 
+bool deleteIfExists(const std::string& file)
+{
+	if (unlink(file.c_str()) != 0)
+		if (errno != ENOENT)
+			throw wibble::exception::File(file, "removing file");
+		else
+			return false;
+	else
+		return true;
+}
+
 Directory::const_iterator Directory::begin()
 {
 	DIR* dir = opendir(m_path.c_str());
